Package: SSIMmap
Title: The Structural Similarity Index Measure for Maps
Version: 0.4.0
Authors@R: c(
  person(given="Hui Jeong (Hailyee)", family="Ha", email="hha24@uwo.ca", role = c("aut", "cre"),comment = c(ORCID = "0000-0002-5229-4528")), 
  person(given="Jed", family="Long", email="jed.long@uwo.ca", role="aut", comment = c(ORCID = "0000-0003-3961-3085")))
Description: Extends the classical SSIM method proposed by 'Wang', 'Bovik', 'Sheikh', and 'Simoncelli'(2004) <doi:10.1109/TIP.2003.819861>. 
  for irregular lattice-based maps and raster images.
  The geographical SSIM method incorporates well-developed 'geographically weighted summary statistics'('Brunsdon', 'Fotheringham' and 'Charlton' 2002) <doi:10.1016/S0198-9715(01)00009-6> 
  with an adaptive bandwidth kernel function for irregular lattice-based maps.
Depends: R (>= 3.5.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: stats, scales, terra, ggplot2, sf, knitr, FNN
Suggests: RColorBrewer, testthat (>= 3.0.0), rmarkdown, patchwork,
        tidyterra
URL: https://github.com/Hailyee-Ha/SSIMmap
BugReports: https://github.com/Hailyee-Ha/SSIMmap/issues
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-07 19:19:37 UTC; hailyeeha
Author: Hui Jeong (Hailyee) Ha [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5229-4528>),
  Jed Long [aut] (ORCID: <https://orcid.org/0000-0003-3961-3085>)
Maintainer: Hui Jeong (Hailyee) Ha <hha24@uwo.ca>
Repository: CRAN
Date/Publication: 2026-05-07 19:40:02 UTC
Built: R 4.5.2; ; 2026-05-07 21:06:55 UTC; unix

Package: SSIMmap
Title: The Structural Similarity Index Measure for Maps
Version: 0.4.0
Authors@R: c(
  person(given="Hui Jeong (Hailyee)", family="Ha", email="hha24@uwo.ca", role = c("aut", "cre"),comment = c(ORCID = "0000-0002-5229-4528")), 
  person(given="Jed", family="Long", email="jed.long@uwo.ca", role="aut", comment = c(ORCID = "0000-0003-3961-3085")))
Description: Extends the classical SSIM method proposed by 'Wang', 'Bovik', 'Sheikh', and 'Simoncelli'(2004) <doi:10.1109/TIP.2003.819861>. 
  for irregular lattice-based maps and raster images.
  The geographical SSIM method incorporates well-developed 'geographically weighted summary statistics'('Brunsdon', 'Fotheringham' and 'Charlton' 2002) <doi:10.1016/S0198-9715(01)00009-6> 
  with an adaptive bandwidth kernel function for irregular lattice-based maps.
Depends: R (>= 3.5.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: stats, scales, terra, ggplot2, sf, knitr, FNN
Suggests: RColorBrewer, testthat (>= 3.0.0), rmarkdown, patchwork,
        tidyterra
URL: https://github.com/Hailyee-Ha/SSIMmap
BugReports: https://github.com/Hailyee-Ha/SSIMmap/issues
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-07 19:19:37 UTC; hailyeeha
Author: Hui Jeong (Hailyee) Ha [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5229-4528>),
  Jed Long [aut] (ORCID: <https://orcid.org/0000-0003-3961-3085>)
Maintainer: Hui Jeong (Hailyee) Ha <hha24@uwo.ca>
Repository: CRAN
Date/Publication: 2026-05-07 19:40:02 UTC
Built: R 4.4.3; ; 2026-05-07 21:51:35 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ssim_bandwidth)
export(ssim_polygon)
export(ssim_raster)
importFrom(FNN,get.knn)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(knitr,kable)
importFrom(scales,rescale)
importFrom(sf,st_centroid)
importFrom(sf,st_coordinates)
importFrom(sf,st_geometry)
importFrom(stats,p.adjust)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(terra,compareGeom)
importFrom(terra,crop)
importFrom(terra,focal)
importFrom(terra,global)
importFrom(terra,ifel)
importFrom(terra,mask)
importFrom(terra,nlyr)
importFrom(terra,values)
importFrom(utils,globalVariables)

# Introduction to SSIMmap

The goal of the _SSIMmap_ package  extend [the classical SSIM method](https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1284395) for irregular lattice-based maps and raster images.The original SSIM method was developed to compare two image mimicking the human visual system. The _SSIMmap_ package applies this method to two types of maps (polygon and raster). The geographical SSIM method incorporates well-developed [geographically weighted summary statistics](https://www.sciencedirect.com/science/article/pii/S0198971501000096?via%3Dihub) with an adaptive bandwidth kernel function for irregular lattice-based maps. This package includes four key functions: **ssim_bandwidth**, **ssim_constant**, **ssim_polygon**,**ssim_raster**. Users who want to compare two maps and quantify their similarities can utilize this package and visualize the results using other R packages(e.g.,[_tmap_](https://CRAN.R-project.org/package=tmap) and [_ggplot_](https://CRAN.R-project.org/package=ggplot2)).


## Installation: SSIMmap

You can install the development version of _SSIMmap_ from [_GitHub_](https://github.com/Hailyee-Ha/SSIMmap) with the following commands:

```
devtools::install_github("Hailyee-Ha/SSIMmap")
library(SSIMmap)
```

## Data: Toronto, groups2nm.tif, and single2nm.tif

The package has three example data: 1) Toronto, 2) groups2nm (*terra raster object*), and 3) single2nm (*terra raster object*). First, **Toronto**(*an object of sf class*) is an example data, which stands for Toronto, ON. This contains neighborhood deprivation indices (the Canadian Index of Multiple Deprivation and Pampalon index) and the census variable (the percentage of households who commute within the census subdivision of residence) for 2016. The second and third data are *terra raster objects*. These raster objects represent the geographical location from 38 to 41◦N and 0.5–5◦E, centering on the islands of Ibiza, Mallorca and Menorca. The raster objects, labeled as 'groups2nm' and 'single2nm',  contain information on sperm whales. They represent the predicted probability of presence for groups and singletons at a spatial resolution of 2 nautical miles (NM) on a regular grid. For further information about 'groups2nm' and 'single2nm' please refer to the paper ["Novel application of a quantitative spatial comparison tool to species distribution data"](https://www.sciencedirect.com/science/article/pii/S1470160X16302990).

```
library(SSIMmap)
library(sf)
library(terra)
shape<- SSIMmap::Toronto
plot(shape, border=NA)
img1<-terra::rast(system.file("data", "groups2nm.tif", package = "SSIMmap"))
img2<-terra::rast(system.file("data", "single2nm.tif", package = "SSIMmap"))
plot(img1)
plot(img2)
```

## Functions: ssim_bandwidth

This function calculates the bandwidth size for the computation of the Structural Similarity Index (SSIM) on polygon maps. A user can decide whether or not to standardize the variables of maps. If the maps have different ranges of variables or include negative values, it is necessary to use **standardize =TRUE**. This ensures that the conditions for SSIM (symmetry and boundedness (a range from -1 to 1)) are not violated. If the maps share a similar range, there is no need to use **standardize= TRUE**.The function provides two options for selecting the bandwidth size, which will be displayed in the console window:

1) The square root of N.
2) The trade-off between the bias and variance of the two maps.

The first method for the bandwidth selection calculates the square root of the sample size n, which is based on the relative rate of convergence that could minimize the mean integrated squared error ([Abramson,1982](https://projecteuclid.org/journals/annals-of-statistics/volume-10/issue-4/On-Bandwidth-Variation-in-Kernel-Estimates-A-Square-Root-Law/10.1214/aos/1176345986.full)). The second method aims to balance local variances and bias in spatial data (geographically weighted mean), with an intent to minimize the difference between them. The primary goal of this method is to find a bandwidth size that balances the bias-variance trade-off and therefore effectively captures the scale of the underlying spatial structure. More information about the bias-variance trade-off can be found in the paper ["On the measurement of bias in geographically weighted regression models"](https://www.sciencedirect.com/science/article/pii/S2211675320300476). By default for the second method, the function chooses the mid-point of the optimal trade-off between the bias and variance for map1 and map2. A user can choose the upper (the larger one) or the lower (the smaller one) value between the two of the optimal trade-off between the bias and variance.

The function takes as input an object of  *sf* class, which includes columns necessary for SSIM calculation. It outputs two possible selections for the bandwidth size based on the selected method. Additionally, the function generates a plot illustrating the relationship between bias, variance, and bandwidth size with a vertical line of the square root of N result.

### How to execute

```
ssim_bandwidth(shape,"CIMD_SDD","PP_SDD",max_bandwidth=80)
ssim_bandwidth(shape,"CIMD_SDD","P_commute",max_bandwidth=80)

```

## Functions: ssim_constant

This function calculates constants (k1 and k2) for the computation of the SSIM index on polygon maps based on the ranges of values. It considers a range of an object of *sf* class including columns for the SSIM calculation and returns the rescaled constants on the console window.

### How to execute

```
ssim_constant(shape,"CIMD_SDD","PP_SDD")
```

## Functions: ssim_polygon

This function calculates the SSIM index for a given polygon. It takes an object of the *sf* class as input, which includes the necessary columns for SSIM calculation. The function then returns either the global SSIM values (when global = TRUE) or the local SSIM values for each given polygon (when global = FALSE). By default, the bandwidth size is determined by the square root of N. A user has the flexibility to select their own bandwidth size or use the suggestions from the *ssim_bandwidth* function. 

### How to execute

```
ssim_polygon(shape,"CIMD_SDD","PP_SDD") 
ssim_polygon(shape,"CIMD_SDD","P_commute") 
df<-ssim_polygon(shape,"CIMD_SDD","PP_SDD",global = FALSE) 
df_2<-ssim_polygon(shape,"CIMD_SDD","P_commute",global = FALSE) 
plot(df,border=NA)
plot(df_2,border=NA)
```

## Functions: ssim_raster

This function calculates the SSIM index for raster images. It takes as input a image file importing from the [_terra_](https://CRAN.R-project.org/package=terra) and returns either the global SSIM values (global=TRUE) or the SSIM values for each given cell as the local SSIM (global=FALSE). Default of the window size is 3\*3 and a user can use own window size.


### How to execute

```
library(rasterVis)
ssim_raster(img1,img2)
result_raster<-ssim_raster(img1,img2,global = FALSE)
levelplot(result_raster)
```

## Example of visualizing the SSIM for irregular lattice maps using ggplot2

```
library(ggplot2)
library(RColorBrewer)

# Visualize the SSIM result (the CIMD vs. Pampalon) on the map
ggplot() +
  geom_sf(data = df, aes(fill = SSIM), color = NA) +
  scale_fill_gradientn(colors = brewer.pal(8, "PRGn"), limits = c(-1, 1)) +
  theme_void() +
  theme(
    axis.title.x = element_blank(),
    axis.title.y = element_blank(),
    axis.text.x = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank()
  )

# Visualize the SSIM result (the CIMD vs. commuting pattern) on the map
ggplot() +
  geom_sf(data = df_2, aes(fill = SSIM), color = NA) +
  scale_fill_gradientn(colors =brewer.pal(8, "PRGn"), limits = c(-1, 1)) +
  theme_void() +
  theme(
    axis.title.x = element_blank(),
    axis.title.y = element_blank(),
    axis.text.x = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank()
  )

```

### Reference

* Brunsdon, C., Fotheringham, A.S. and Charlton, M. (2002). Geographically weighted summary statistics---a framework for localised exploratory data analysis. *Computers, Environment and Urban Systems*, 26(6), pp.501-524. https://doi.org/10.1016/S0198-9715(01)00009-6
* Jones, E.L., Rendell, L., Pirotta, E. and Long, J.A. (2016). Novel application of a quantitative spatial comparison tool to species distribution data. *Ecological Indicators*, 70, pp.67-76. https://doi.org/10.1016/j.ecolind.2016.05.051
* Lu, B., Harris, P., Charlton, M. and Brunsdon, C. (2014). The GWmodel R package: further topics for exploring spatial heterogeneity using geographically weighted models. *Geo-spatial Information Science*, 17(2), pp.85-101. https://doi.org/10.1080/10095020.2014.917453
* Wang, Z., Bovik, A.C., Sheikh, H.R. and Simoncelli, E.P. (2004). Image quality assessment: from error visibility to structural similarity. *IEEE transactions on image processing*, 13(4), pp.600-612. https://doi.org/10.1109/TIP.2003.819861

## ----setup, echo = FALSE------------------------------------------------------
suppressPackageStartupMessages(library(SSIMmap))
knitr::opts_chunk$set(warning = FALSE, message = FALSE) 

## ----message = FALSE----------------------------------------------------------
library(SSIMmap)
library(sf)
library(terra)
library(ggplot2)
library(patchwork)
library(tidyterra)

# Polygon data
data("Toronto_SSIM")
plot(Toronto_SSIM["CIMD"], border = NA, main = "CIMD")

# Raster data (FWI for British Columbia)
# Stored as PackedSpatRaster — unwrap to SpatRaster for use with terra
data("fwi_0816_bc"); fwi_0816_bc <- terra::unwrap(fwi_0816_bc)
data("fwi_0818_bc"); fwi_0818_bc <- terra::unwrap(fwi_0818_bc)
data("fwi_1101_bc"); fwi_1101_bc <- terra::unwrap(fwi_1101_bc)

plot(fwi_0816_bc, main = "FWI – 16 August 2023")
plot(fwi_0818_bc, main = "FWI – 18 August 2023")
plot(fwi_1101_bc, main = "FWI – 1 November 2023")

## -----------------------------------------------------------------------------
args(ssim_bandwidth)

## ----fig.width = 6, fig.height = 4--------------------------------------------
S1 <- ssim_bandwidth(
  shape         = Toronto_SSIM,
  map1          = "CIMD",
  map2          = "PP",
  max_bandwidth = 100,
  transform     = "percentile",
  option        = "midpoint"
)
S1$bandwidth
S1$plot

## ----fig.width = 6, fig.height = 4--------------------------------------------
S2 <- ssim_bandwidth(
  shape         = Toronto_SSIM,
  map1          = "CIMD",
  map2          = "Commute",
  max_bandwidth = 100,
  transform     = "percentile",
  option        = "midpoint"
)
S2$bandwidth
S2$plot

## ----fig.width = 10, fig.height = 5-------------------------------------------
# Tag each panel and remove individual captions
p1 <- S1$plot +
  labs(tag = "a", caption = NULL) +
  theme(plot.tag = element_text(face = "bold", size = 14))

p2 <- S2$plot +
  labs(tag = "b", caption = NULL) +
  theme(plot.tag = element_text(face = "bold", size = 14))

# Combine side-by-side with a shared legend at the bottom
combined_bandwidth_plot <- (p1 + p2) +
  plot_layout(guides = "collect") &
  theme(legend.position = "bottom")

# Add a single global caption
combined_bandwidth_plot +
  plot_annotation(
    caption = "* Solid line: Bias / Dashed line: Variance",
    theme   = theme(plot.caption = element_text(size = 10, hjust = 0.5))
  )

## -----------------------------------------------------------------------------
args(ssim_polygon)

## -----------------------------------------------------------------------------
S1_global <- ssim_polygon(
  shape     = Toronto_SSIM,
  map1      = "CIMD",
  map2      = "PP",
  global    = TRUE,
  bandwidth = 87,
  transform = "percentile",
  k1 = NULL, k2 = NULL,
  do_test = TRUE, R = 1000, fdr = TRUE, alpha = 0.05,
  seed = 1
)

# Global means and permutation p-values
S1_global$p_global

## -----------------------------------------------------------------------------
S2_global <- ssim_polygon(
  shape     = Toronto_SSIM,
  map1      = "CIMD",
  map2      = "Commute",
  global    = TRUE,
  bandwidth = 91,
  transform = "percentile",
  do_test = TRUE, R = 1000, fdr = TRUE, alpha = 0.05,
  seed = 1
)
S2_global$p_global

## -----------------------------------------------------------------------------
S1_local <- ssim_polygon(
  shape     = Toronto_SSIM,
  map1      = "CIMD",
  map2      = "PP",
  global    = FALSE,
  bandwidth = 87,
  transform = "percentile",
  do_test = TRUE, R = 1000, fdr = TRUE, alpha = 0.05,
  seed = 1
)
head(S1_local[, c("SSIM", "SIM", "SIV", "SIP", "p_value", "q_value", "sig")])

## -----------------------------------------------------------------------------
S2_local <- ssim_polygon(
  shape     = Toronto_SSIM,
  map1      = "CIMD",
  map2      = "Commute",
  global    = FALSE,
  bandwidth = 91,
  transform = "percentile",
  do_test = TRUE, R = 1000, fdr = TRUE, alpha = 0.05,
  seed = 1
)

## ----fig.show = "hold", out.width = "45%"-------------------------------------
library(RColorBrewer)

# CIMD vs. Pampalon
ggplot() +
  geom_sf(data = S1_local, aes(fill = SSIM), color = NA) +
  scale_fill_gradientn(colors = brewer.pal(8, "PRGn"), limits = c(-1, 1)) +
  theme_void()

# CIMD vs. Commute
ggplot() +
  geom_sf(data = S2_local, aes(fill = SSIM), color = NA) +
  scale_fill_gradientn(colors = brewer.pal(8, "PRGn"), limits = c(-1, 1)) +
  theme_void()

## -----------------------------------------------------------------------------
args(ssim_raster)

## -----------------------------------------------------------------------------
ssim_raster(fwi_0816_bc, fwi_0818_bc)   # summer vs summer (high similarity expected)
ssim_raster(fwi_0816_bc, fwi_1101_bc)   # summer vs fall   (lower similarity expected)

## ----eval = FALSE-------------------------------------------------------------
# # Comparison 1: summer vs summer
# p1_l <- ssim_raster(
#   fwi_0816_bc, fwi_0818_bc,
#   global     = FALSE,
#   w          = 1,
#   do_test    = TRUE,
#   local_test = TRUE,
#     fdr= TRUE,
#   R          = 999
# )
# 
# # Comparison 2: summer vs fall
# p2_l <- ssim_raster(
#   fwi_0816_bc, fwi_1101_bc,
#   global     = FALSE,
#   w          = 1,
#   do_test    = TRUE,
#   local_test = TRUE,
#   fdr= TRUE,
#   R          = 999
# )

## ----eval = FALSE-------------------------------------------------------------
# crs_albers <- "EPSG:3005"
# 
# # Re-project local SSIM result (continuous values → bilinear)
# p1_l_alb <- terra::project(p1_l, crs_albers, method = "bilinear")
# 
# # Select SSIM + components and assign panel labels
# ssim_maps_alb <- p1_l_alb[[c("SSIM", "SIM", "SIV", "SIP")]]
# facet_labs <- c(SSIM = "a", SIM = "b", SIV = "c", SIP = "d")
# 
# gg <- ggplot() +
#   geom_spatraster(data = ssim_maps_alb) +
#   facet_wrap(~lyr, ncol = 2, labeller = labeller(lyr = facet_labs)) +
#   scale_fill_viridis_c(
#     limits   = c(-1, 1),
#     na.value = "transparent",
#     name     = "Values"
#   ) +
#   coord_sf(crs = crs_albers, expand = FALSE) +
#   theme_minimal() +
#   theme(
#     strip.text       = element_text(size = 14, face = "bold", hjust = 0),
#     strip.background = element_blank(),
#     legend.title     = element_text(size = 13, face = "bold"),
#     legend.text      = element_text(size = 11),
#     panel.background = element_blank(),
#     panel.grid       = element_blank()
#   )
# gg

## ----eval = FALSE-------------------------------------------------------------
# # Re-project the second comparison and keep only SSIM
# p2_l_alb         <- terra::project(p2_l, crs_albers, method = "bilinear")
# ssim_maps_alb_p2 <- p2_l_alb[[c("SSIM")]]
# 
# ssim_maps_alb_p2 <- p2_l_alb[[c("SSIM", "SIM", "SIV", "SIP")]]
# facet_labs <- c(SSIM = "a", SIM = "b", SIV = "c", SIP = "d")
# 
# gg2 <- ggplot() +
#   geom_spatraster(data = ssim_maps_alb_p2) +
#   facet_wrap(~lyr, ncol = 2, labeller = labeller(lyr = facet_labs)) +
#   scale_fill_viridis_c(
#     limits   = c(-1, 1),
#     na.value = "transparent",
#     name     = "Values"
#   ) +
#   coord_sf(crs = crs_albers, expand = FALSE) +
#   theme_minimal() +
#   theme(
#     strip.text       = element_text(size = 14, face = "bold", hjust = 0),
#     strip.background = element_blank(),
#     legend.title     = element_text(size = 13, face = "bold"),
#     legend.text      = element_text(size = 11),
#     panel.background = element_blank(),
#     panel.grid       = element_blank())
# gg2

## ----eval = FALSE-------------------------------------------------------------
# p_maps <- p1_l[[c("SSIM_p", "SIM_p", "SIV_p", "SIV_p")]]
# plot(p_maps,
#      main = c("p-value: SSIM", "p-value: SIM",
#               "p-value: SIV",  "p-value: SIP"))

#' Bandwidth selection for polygon maps
#'
#' Computes bias–variance trade-off curves for two polygon maps using an
#' adaptive Gaussian kernel. The two input variables can be optionally
#' transformed (e.g. normal scores or min–max normalisation) before smoothing.
#' A suggested bandwidth range is obtained by balancing standardized bias and
#' variance for each variable, and a single bandwidth is chosen from this range.
#'
#' This function does *not* compute SSIM; it is intended to be used in
#' combination with \code{ssim_polygon()}, which can then be called with the
#' selected bandwidth.
#'
#' @param shape An \code{sf} polygon object with attributes containing the two
#'   variables of interest.
#' @param map1 Character string; column name in \code{shape} for the first map.
#' @param map2 Character string; column name in \code{shape} for the second map.
#' @param max_bandwidth Integer (at least 12). Upper bound of the bandwidth
#'   \code{k} for the adaptive k-nearest-neighbour kernel. Must not exceed the
#'   number of polygons.
#' @param transform One of \code{c("normal_score","percentile","none","minmax")}.
#'   \code{"normal_score"} applies a Blom normal scores transform;
#'   \code{"percentile"} maps values to empirical percentiles in (0, 1);
#'   \code{"minmax"} applies min–max normalisation to [0, 1]; and
#'   \code{"none"} leaves the variables on their original scale.
#' @param option Character string specifying how to choose a single bandwidth
#'   from the bias–variance trade-off range: one of
#'   \code{"midpoint"}, \code{"lower"}, or \code{"upper"} (default
#'   \code{"midpoint"}). The square-root of the number of polygons (\code{sqrt(n)})
#'   is also shown on the plot as a reference.
#'
#' @return A list with components:
#'   \item{plot}{A \code{ggplot} object showing the bias–variance trade-off
#'     curves for both variables, the suggested bandwidth range, the chosen
#'     bandwidth, and \code{sqrt(n)} as a reference.}
#'   \item{bandwidth}{The chosen bandwidth \code{k_star}.}
#'   \item{tradeoff}{A list with elements
#'     \code{map1}, \code{map2}, \code{kA}, \code{kB}, and \code{k_sqrt},
#'     where \code{map1} and \code{map2} are data frames containing the
#'     standardized bias and variance across \code{k} for each variable.}
#'
#' @details
#' The kernel is Gaussian with adaptive bandwidth:
#' \deqn{w_{ij} = \exp\left\{-\tfrac12 (d_{ij} / h_i)^2 \right\},}
#' where \eqn{d_{ij}} is the distance between polygon centroids and
#' \eqn{h_i} is the distance from polygon \eqn{i} to its \eqn{k}-th nearest
#' neighbour. Bandwidth selection is based purely on bias–variance trade-off
#' and does not involve any SSIM computation or permutation testing.
#'
#' @importFrom sf st_geometry st_centroid st_coordinates
#' @importFrom FNN get.knn
#' @importFrom ggplot2 ggplot geom_line geom_rect geom_vline labs geom_text aes
#' @importFrom stats qnorm var
#' @importFrom scales rescale
#' @importFrom utils globalVariables
#' @export
ssim_bandwidth <- function(
    shape, map1, map2,
    max_bandwidth,
    transform = c("normal_score","percentile","none","minmax"),
    option = "midpoint"
){
  ## ---------- checks ----------
  transform <- match.arg(transform)
  if (map1 == map2) stop("variables are identical")
  if (max_bandwidth < 12) stop("max_bandwidth must be >= 12")
  n <- nrow(shape)
  if (n < max_bandwidth) stop("max_bandwidth is larger than number of polygons (n)")

  ## ---------- helpers ----------
  normal_scores <- function(x, ties = "average"){
    # Blom normal scores transformation
    r <- rank(x, ties.method = ties); n <- length(x)
    u <- (r - 3/8) / (n + 1/4)
    qnorm(u)
  }

  to_percentile <- function(x, ties="average"){
    # Map to (0,1) empirical percentiles
    rank(x, ties.method = ties)/(length(x)+1)
  }

  to_minmax <- function(x){
    # Standard min-max normalization to [0,1]
    rng <- range(x, na.rm = TRUE)
    if (!all(is.finite(rng)) || rng[1] == rng[2]) {
      stop("Min-max normalization is not defined when all values are identical or non-finite.")
    }
    (x - rng[1]) / (rng[2] - rng[1])
  }

  ## ---------- extract & transform ----------
  dat <- as.data.frame(shape)
  xA <- dat[[map1]]
  xB <- dat[[map2]]

  xA_t <- switch(
    transform,
    normal_score = normal_scores(xA),
    percentile   = to_percentile(xA),
    none         = xA,
    minmax       = to_minmax(xA)
  )

  xB_t <- switch(
    transform,
    normal_score = normal_scores(xB),
    percentile   = to_percentile(xB),
    none         = xB,
    minmax       = to_minmax(xB)
  )

  ## ---------- geometry & k-NN (for all k we reuse the same neighbour order) ----------
  coords <- sf::st_coordinates(sf::st_centroid(sf::st_geometry(shape)))
  nn <- FNN::get.knn(coords, k = max_bandwidth)

  ## ---------- local weighted mean for a given k ----------
  lw_mean <- function(x, k){
    idx  <- nn$nn.index[, 1:k, drop=FALSE]
    dist <- nn$nn.dist[,  1:k, drop=FALSE]
    hi   <- apply(dist, 1, max)
    W    <- exp(-0.5 * (dist / hi)^2)     # Gaussian kernel weights
    num  <- rowSums(W * x[idx])
    den  <- rowSums(W)
    num / den
  }

  ## ---------- bias-variance curves over k ----------
  bw_seq <- 12:max_bandwidth

  LM_A <- sapply(bw_seq, function(k) lw_mean(xA_t, k))
  LM_B <- sapply(bw_seq, function(k) lw_mean(xB_t, k))

  # Bias: mean squared deviation between local mean and original value
  Bias_A <- colMeans((LM_A - xA_t)^2)
  Bias_B <- colMeans((LM_B - xB_t)^2)

  # Variance: variance across polygons of the local mean surface
  Var_A  <- apply(LM_A, 2, var)
  Var_B  <- apply(LM_B, 2, var)

  # standardize to [0,1] for trade-off comparison
  Bias_A_s <- scales::rescale(Bias_A)
  Bias_B_s <- scales::rescale(Bias_B)
  Var_A_s  <- scales::rescale(Var_A)
  Var_B_s  <- scales::rescale(Var_B)

  trade_A <- data.frame(Bandwidth = bw_seq, Bias = Bias_A_s, Variance = Var_A_s)
  trade_B <- data.frame(Bandwidth = bw_seq, Bias = Bias_B_s, Variance = Var_B_s)

  # bandwidth where |Bias - Var| is minimized for each map
  kA <- trade_A$Bandwidth[ which.min(abs(trade_A$Bias - trade_A$Variance)) ]
  kB <- trade_B$Bandwidth[ which.min(abs(trade_B$Bias - trade_B$Variance)) ]

  k_sqrt     <- ceiling(sqrt(n))
  k_rect_min <- min(kA, kB)
  k_rect_max <- max(kA, kB)

  # choose bandwidth
  k_star <-
    if (option == "midpoint") ceiling(mean(c(kA, kB))) else
      if (option == "lower")  k_rect_min else
        if (option == "upper")  k_rect_max else
          stop("option must be one of 'midpoint', 'lower', 'upper'")

  ## ---------- plot (bias-variance trade-off) ----------
  p <- ggplot2::ggplot() +
    # Variable A: Green (Solid for Bias, Dashed for Variance)
    ggplot2::geom_line(data = trade_A,
                       ggplot2::aes(Bandwidth, Bias, color = "Variable A"),
                       linewidth = 0.8) +
    ggplot2::geom_line(data = trade_A,
                       ggplot2::aes(Bandwidth, Variance, color = "Variable A"),
                       linetype = "dashed", linewidth = 0.8) +

    # Variable B: Blue (Solid for Bias, Dashed for Variance)
    ggplot2::geom_line(data = trade_B,
                       ggplot2::aes(Bandwidth, Bias, color = "Variable B"),
                       linewidth = 0.8) +
    ggplot2::geom_line(data = trade_B,
                       ggplot2::aes(Bandwidth, Variance, color = "Variable B"),
                       linetype = "dashed", linewidth = 0.8) +

    # Suggested Range and Reference Lines
    ggplot2::geom_rect(
      ggplot2::aes(xmin = k_rect_min, xmax = k_rect_max, ymin = 0, ymax = 1),
      fill = "grey70", alpha = 0.3
    ) +
    ggplot2::geom_vline(xintercept = k_sqrt, linewidth = 0.5, color = "grey40") +
    ggplot2::geom_vline(xintercept = c(kA, kB), color = "red",
                        linewidth = 0.3, alpha = 0.6) +
    ggplot2::geom_vline(xintercept = k_star, color = "purple",
                        linewidth = 0.8) +

    # Labels and Scales
    ggplot2::geom_text(
      ggplot2::aes(x = k_sqrt, y = 1.02),
      label = paste0("sqrt(n)=", k_sqrt),
      size = 3.5, color = "grey40"
    ) +
    ggplot2::scale_color_manual(
      name = "Variables",
      values = c("Variable A" = "forestgreen", "Variable B" = "royalblue")
    ) +
    ggplot2::labs(
      x = "Bandwidth (k-nearest neighbours)",
      y = "Standardized bias/variance",
      subtitle = paste0("Range [", k_rect_min, ", ", k_rect_max, "]; Chosen = ", k_star),
      caption = "* Solid line: Bias / Dashed line: Variance"
    ) +
    ggplot2::theme_minimal()

  out <- list(
    plot = p,
    bandwidth = k_star,
    tradeoff = list(
      map1   = trade_A,
      map2   = trade_B,
      kA     = kA,
      kB     = kB,
      k_sqrt = k_sqrt
    )
  )

  return(out)
}
utils::globalVariables(c("Bandwidth", "Bias", "Variance"))

#' Structural similarity index (SSIM) for polygon maps
#'
#' Computes local and global SSIM (and its components SIM, SIV, SIP) for two
#' polygon maps using an adaptive k-nearest-neighbour (k-NN) Gaussian kernel.
#' Input variables can be optionally transformed (e.g. rank-based inverse
#' normal scores or min–max normalization). 
#' Optionally performs permutation tests for global and local significance with BH-FDR correction.
#' Under the null hypothesis for the permutation test, both variables are
#' randomly permuted at each iteration, breaking spatial structure and
#' association between them. Local p-values can be optionally adjusted using
#' the Benjamini–Hochberg FDR procedure.
#'
#' @param shape An \code{sf} polygon object.
#' @param map1 Character string; column name in \code{shape} for the first map.
#' @param map2 Character string; column name in \code{shape} for the second map.
#' @param global Logical. If \code{TRUE}, compute and print a summary table of
#'   global SSIM, SIM, SIV, SIP. If \code{FALSE}, return an \code{sf} object
#'   with local metrics (and local p-/q-values if \code{do_test = TRUE}).
#' @param bandwidth Integer; adaptive k-NN size (number of neighbours). The
#'   default is \code{ceiling(sqrt(n))}, where \code{n} is the number of
#'   polygons. Must be at least 3 and not exceed \code{n}.
#' @param transform One of \code{c("normal_score", "percentile", "none", "minmax")}.
#'   \code{"normal_score"} applies a Blom normal scores transform;
#'   \code{"percentile"} maps values to empirical percentiles in (0, 1);
#'   \code{"minmax"} applies min–max normalisation to [0, 1]; and
#'   \code{"none"} leaves the variables on their original scale.
#' @param k1,k2 SSIM constants. If \code{NULL}, defaults are \code{k1 = 0.01},
#'   \code{k2 = 0.03}.
#' @param do_test Logical; if \code{TRUE}, perform permutation tests for global
#'   and local SSIM. Local p-values can be FDR-adjusted if \code{fdr = TRUE}.
#' @param R Integer; number of permutations for the significance tests
#'   (default \code{R = 1000}).
#' @param fdr Logical; if \code{TRUE} (default), apply Benjamini–Hochberg
#'   FDR correction to local p-values.
#' @param alpha Numeric; significance threshold for local results (default
#'   \code{0.05}). Local features with \code{q_value < alpha} are flagged as
#'   significant.
#' @param seed Optional integer; random seed for reproducibility of the
#'   permutation tests.
#'
#' @return
#'   If \code{global = TRUE} and \code{do_test = FALSE}, the function prints a
#'   knitr table summarising the global mean, minimum, maximum, and standard
#'   deviation of SSIM, SIM, SIV, and SIP, and returns this \code{data.frame}
#'   (invisibly).
#'
#'   If \code{global = TRUE} and \code{do_test = TRUE}, the function prints the
#'   same summary table plus a table of global permutation p-values (two-sided)
#'   for SSIM, SIM, SIV, and SIP, and returns (invisibly) a \code{list} with
#'   components:
#'   \itemize{
#'     \item \code{summary}: \code{data.frame} with global SSIM/SIM/SIV/SIP
#'       summary statistics;
#'     \item \code{p_global}: \code{data.frame} with global means and
#'       permutation p-values for SSIM, SIM, SIV, SIP.
#'   }
#'
#'   If \code{global = FALSE}, the function returns an \code{sf} object equal to
#'   \code{shape} with additional columns:
#'   \itemize{
#'     \item \code{SSIM}, \code{SIM}, \code{SIV}, \code{SIP}: local similarity
#'       metrics for each polygon;
#'     \item \code{p_value}, \code{q_value}, \code{sig} (only if
#'       \code{do_test = TRUE}): local permutation p-value, FDR-adjusted q-value,
#'       and a logical flag indicating significance \code{(q_value < alpha)}.
#'   }
#'
#' @importFrom sf st_geometry st_centroid st_coordinates
#' @importFrom FNN get.knn
#' @importFrom stats qnorm sd quantile p.adjust
#' @importFrom knitr kable
#'
#' @export
#'
ssim_polygon <- function(
    shape, map1, map2,
    global = TRUE,
    bandwidth = NULL,
    transform = c("normal_score","percentile","none","minmax"),
    k1 = NULL, k2 = NULL,
    do_test = FALSE, R = 1000, fdr = TRUE, alpha = 0.05,
    seed = NULL
){
  transform <- match.arg(transform)
  if (identical(map1, map2)) stop("variables are identical")
  if (!map1 %in% names(shape) || !map2 %in% names(shape))
    stop("map1/map2 must be column names in 'shape'")
  
  n <- nrow(shape)
  if (is.null(bandwidth)) bandwidth <- ceiling(sqrt(n))
  if (bandwidth < 3) stop("bandwidth must be >= 3")
  if (bandwidth > n) stop("bandwidth cannot exceed number of polygons")
  
  if (!is.null(seed)) set.seed(seed)
  
  # --- helpers: transforms ---
  normal_scores <- function(x, ties="average"){
    # Blom normal scores transformation
    r <- rank(x, ties.method = ties); n <- length(x)
    u <- (r - 3/8) / (n + 1/4)
    qnorm(u)
  }
  
  to_percentile <- function(x, ties="average"){
    # Map to (0,1) empirical percentiles
    rank(x, ties.method = ties)/(length(x)+1)
  }
  
  to_minmax <- function(x){
    # Standard min-max normalization to [0,1]
    rng <- range(x, na.rm = TRUE)
    if (!all(is.finite(rng)) || rng[1] == rng[2]) {
      stop("Min-max normalization is not defined when all values are identical or non-finite.")
    }
    (x - rng[1]) / (rng[2] - rng[1])
  }
  
  # --- extract & transform variables ---
  A_raw <- as.numeric(shape[[map1]])
  B_raw <- as.numeric(shape[[map2]])
  
  A <- switch(
    transform,
    normal_score = normal_scores(A_raw),
    percentile   = to_percentile(A_raw),
    none         = A_raw,
    minmax       = to_minmax(A_raw)
  )
  
  B <- switch(
    transform,
    normal_score = normal_scores(B_raw),
    percentile   = to_percentile(B_raw),
    none         = B_raw,
    minmax       = to_minmax(B_raw)
  )
  
  # --- geometry & neighbours (adaptive k-NN) ---
  coords <- sf::st_coordinates(sf::st_centroid(sf::st_geometry(shape)))
  knn <- FNN::get.knn(coords, k = bandwidth)
  idx  <- knn$nn.index
  dist <- knn$nn.dist
  hi   <- apply(dist, 1, max)               # distance to k-th neighbour
  W    <- exp(-0.5 * (dist / hi)^2)         # Gaussian kernel
  Wn   <- W / rowSums(W)                    # row-normalize weights
  
  # --- local stats given A,B ---
  lw_stats_two <- function(A, B){
    mA <- rowSums(Wn * A[idx]); mB <- rowSums(Wn * B[idx])
    # local variances (expectation of squared deviations with normalized W)
    vA <- rowSums(Wn * (A[idx] - mA)^2)
    vB <- rowSums(Wn * (B[idx] - mB)^2)
    cAB<- rowSums(Wn * (A[idx] - mA) * (B[idx] - mB))
    list(mA=mA, mB=mB, vA=vA, vB=vB, cAB=cAB)
  }
  st_obs <- lw_stats_two(A, B)
  
  # --- SSIM constants (robust) ---
  if (is.null(k1)) k1 <- 0.01
  if (is.null(k2)) k2 <- 0.03
  if (k1 <= 0 || k2 <= 0 || k1 >= 1 || k2 >= 1) stop("k1/k2 must be in (0,1)")
  
  both <- c(A, B)
  Rrob <- as.numeric(
    quantile(both, 0.99, na.rm=TRUE) - quantile(both, 0.01, na.rm=TRUE)
  )
  if (!is.finite(Rrob) || Rrob <= 0) Rrob <- diff(range(both, na.rm=TRUE))
  C1 <- (k1 * Rrob)^2; C2 <- (k2 * Rrob)^2; C3 <- C2/2
  
  # --- components (raw) ---
  SIM_raw <- (2*st_obs$mA*st_obs$mB + C1) /
    (st_obs$mA^2 + st_obs$mB^2 + C1)
  SIV_raw <- (2*sqrt(st_obs$vA)*sqrt(st_obs$vB) + C2) /
    (st_obs$vA + st_obs$vB + C2)
  SIP_raw <- (st_obs$cAB + C3) /
    (sqrt(st_obs$vA)*sqrt(st_obs$vB) + C3)
  
  tol <- 1e-8
  
  # Hard range check: if any metric is clearly outside its theoretical range,
  # stop and ask the user to try a different transform option.
  check_range <- function(x, lo, hi, name){
    if (any(x < lo - tol | x > hi + tol, na.rm = TRUE)) {
      stop(
        sprintf(
          "%s values are outside their theoretical range [%g, %g]. This may indicate numerical instability. Please try a different 'transform' option.",
          name, lo, hi
        )
      )
    }
  }
  
  check_range(SIM_raw, 0, 1,  "SIM")
  check_range(SIV_raw, 0, 1,  "SIV")
  check_range(SIP_raw, -1, 1, "SIP")
  
  SIM <- SIM_raw
  SIV <- SIV_raw
  SIP <- SIP_raw
  SSIM <- SIM * SIV * SIP
  
  if (global && !do_test){
    res <- data.frame(
      Statistic = c("Mean","Min","Max","SD"),
      SSIM = c(mean(SSIM), min(SSIM), max(SSIM), sd(SSIM)),
      SIM  = c(mean(SIM ), min(SIM ), max(SIM ), sd(SIM )),
      SIV  = c(mean(SIV ), min(SIV ), max(SIV ), sd(SIV )),
      SIP  = c(mean(SIP ), min(SIP ), max(SIP ), sd(SIP ))
    )
    print(knitr::kable(res, row.names = FALSE))
    invisible(res)
  } else {
    p_loc <- q_loc <- sig <- NULL
    
    if (do_test){
      S_g_null   <- numeric(R)
      SIM_g_null <- numeric(R)
      SIV_g_null <- numeric(R)
      SIP_g_null <- numeric(R)
      S_i_null   <- matrix(NA_real_, nrow = n, ncol = R)
      
      for (r in seq_len(R)){
        
        # ---- PERMUTATION STEP (UPDATED) ----
        # Under the null hypothesis, both A and B are randomly permuted.
        # This breaks any spatial structure and association between them.
        Ap <- sample(A, replace = FALSE)
        Bp <- sample(B, replace = FALSE)
        
        st  <- lw_stats_two(Ap, Bp)
        
        SIMp_raw <- (2*st$mA*st$mB + C1) /
          (st$mA^2 + st$mB^2 + C1)
        SIVp_raw <- (2*sqrt(st$vA)*sqrt(st$vB) + C2) /
          (st$vA + st$vB + C2)
        SIPp_raw <- (st$cAB + C3) /
          (sqrt(st$vA)*sqrt(st$vB) + C3)
        
        # Range check also applied to permutation metrics
        check_range(SIMp_raw, 0, 1,  "SIM (permutation)")
        check_range(SIVp_raw, 0, 1,  "SIV (permutation)")
        check_range(SIPp_raw, -1, 1, "SIP (permutation)")
        
        SIMp <- SIMp_raw
        SIVp <- SIVp_raw
        SIPp <- SIPp_raw
        SSi  <- SIMp * SIVp * SIPp
        
        SIM_g_null[r] <- mean(SIMp)
        SIV_g_null[r] <- mean(SIVp)
        SIP_g_null[r] <- mean(SIPp)
        S_g_null[r]   <- mean(SSi)
        
        S_i_null[, r] <- SSi
      }
      
      # Observed global means
      SIM_g  <- mean(SIM); SIV_g <- mean(SIV); SIP_g <- mean(SIP); SSIM_g <- mean(SSIM)
      
      pval_two_sided <- function(null_vec, obs_mean){
        mu <- mean(null_vec)
        (sum(abs(null_vec - mu) >= abs(obs_mean - mu)) + 1) / (length(null_vec) + 1)
      }
      p_global_SSIM <- pval_two_sided(S_g_null,   SSIM_g)
      p_global_SIM  <- pval_two_sided(SIM_g_null, SIM_g)
      p_global_SIV  <- pval_two_sided(SIV_g_null, SIV_g)
      p_global_SIP  <- pval_two_sided(SIP_g_null, SIP_g)
      
      # Local SSIM p / q
      mu_i  <- rowMeans(S_i_null)
      p_loc <- (rowSums(abs(S_i_null - mu_i) >= abs(SSIM - mu_i)) + 1) / (R + 1)
      q_loc <- if (fdr) p.adjust(p_loc, method = "BH") else p_loc
      sig   <- q_loc < alpha
    }
    
    # ------ return branch ------
    if (global) {
      res <- data.frame(
        Statistic = c("Mean","Min","Max","SD"),
        SSIM = c(mean(SSIM), min(SSIM), max(SSIM), sd(SSIM)),
        SIM  = c(mean(SIM ), min(SIM ), max(SIM ), sd(SIM )),
        SIV  = c(mean(SIV ), min(SIV ), max(SIV ), sd(SIV )),
        SIP  = c(mean(SIP ), min(SIP ), max(SIP ), sd(SIP ))
      )
      print(knitr::kable(res, row.names = FALSE))
      
      if (do_test){
        res_p <- data.frame(
          Component   = c("SSIM","SIM","SIV","SIP"),
          Global_Mean = c(SSIM_g, SIM_g, SIV_g, SIP_g),
          P_value     = c(p_global_SSIM, p_global_SIM, p_global_SIV, p_global_SIP)
        )
        cat("\nGlobal permutation p-values (two-sided):\n")
        print(knitr::kable(res_p, row.names = FALSE, digits = 4))
        return(invisible(list(summary = res, p_global = res_p)))
      } else {
        return(invisible(res))
      }
      
    } else {
      # global == FALSE → always return sf with local metrics
      out_sf <- shape
      out_sf$SSIM <- SSIM
      out_sf$SIM  <- SIM
      out_sf$SIV  <- SIV
      out_sf$SIP  <- SIP
      if (do_test){
        out_sf$p_value <- p_loc
        out_sf$q_value <- q_loc
        out_sf$sig     <- sig
      }
      return(out_sf)
    }
  }
}

#' SSIM index for raster images
#'
#' This function calculates the Structural Similarity Index Measure (SSIM)
#' between two raster images. It can return either global SSIM summaries or
#' local SSIM raster layers. Optional permutation-based statistical inference
#' can also be performed for global or local SSIM values.
#'
#' @param map1 A single-layer \code{terra::SpatRaster} object representing the first raster map.
#' @param map2 A single-layer \code{terra::SpatRaster} object representing the second raster map.
#' @param global Logical. If \code{TRUE}, the function returns global summary
#'   statistics of SSIM, SIM, SIV, and SIP. If \code{FALSE}, it returns local
#'   raster layers for each cell. Default is \code{TRUE}.
#' @param w Integer. Radius of the local moving window used to calculate local
#'   SSIM components. For example, \code{w = 1} uses a 3 x 3 window, and
#'   \code{w = 2} uses a 5 x 5 window. Default is \code{1}.
#' @param transform Character. Transformation applied to both raster maps before
#'   SSIM calculation. Options are \code{"normal_score"}, \code{"percentile"},
#'   \code{"none"}, and \code{"minmax"}. Default is \code{"normal_score"}.
#' @param k1 Numeric. Constant used to stabilize the luminance component of SSIM.
#'   If \code{NULL}, the default value \code{0.01} is used.
#' @param k2 Numeric. Constant used to stabilize the contrast and structure
#'   components of SSIM. If \code{NULL}, the default value \code{0.03} is used.
#' @param do_test Logical. If \code{TRUE}, a permutation-based statistical test
#'   is performed. Default is \code{FALSE}.
#' @param local_test Logical. If \code{TRUE} and \code{global = FALSE}, local
#'   permutation tests are performed for each raster cell. If \code{FALSE},
#'   local SSIM layers are returned without p-values. Default is \code{FALSE}.
#' @param R Integer. Number of permutations used for statistical inference.
#'   Default is \code{1000}.
#' @param fdr Logical. If \code{TRUE}, p-values are adjusted using the
#'   Benjamini-Hochberg false discovery rate correction. Default is \code{TRUE}.
#' @param alpha Numeric. Significance level used to determine significant SSIM
#'   values. Default is \code{0.05}.
#' @param seed Optional integer. Random seed used for permutation testing.
#'   Default is \code{NULL}.
#'
#' @return
#' If \code{global = TRUE} and \code{do_test = FALSE}, a list containing a summary
#' table of global SSIM, SIM, SIV, and SIP values is returned.
#'
#' If \code{global = TRUE} and \code{do_test = TRUE}, a list containing the
#' summary table, permutation null distributions, p-values, and optionally
#' q-values is returned.
#'
#' If \code{global = FALSE} and \code{do_test = FALSE}, a
#' \code{terra::SpatRaster} object with four layers is returned:
#' \code{SSIM}, \code{SIM}, \code{SIV}, and \code{SIP}.
#'
#' If \code{global = FALSE}, \code{do_test = TRUE}, and
#' \code{local_test = TRUE}, a \code{terra::SpatRaster} object is returned with
#' local SSIM component layers, p-value layers, optionally q-value layers, and
#' significance layers.
#'
#' @details
#' The SSIM index is calculated as the product of three components:
#' similarity in mean intensity, similarity in variance, and similarity in
#' spatial pattern. These components are returned as \code{SIM}, \code{SIV}, and
#' \code{SIP}, respectively.
#'
#' The local SSIM calculation uses a square moving window with size
#' \code{(2 * w + 1) x (2 * w + 1)}.
#'
#' When \code{do_test = TRUE}, permutation testing is performed by randomly
#' permuting the values of \code{map2} over the overlapping non-\code{NA} cells
#' and recalculating SSIM values.
#'
#' @examples
#' \dontrun{
#' library(terra)
#'
#' # Global SSIM summary
#' g1 <- ssim_raster(
#'   map1,
#'   map2,
#'   global = TRUE,
#'   w = 1
#' )
#'
#' # Local SSIM raster layers
#' l1 <- ssim_raster(
#'   map1,
#'   map2,
#'   global = FALSE,
#'   w = 1
#' )
#'
#' # Local SSIM with permutation test
#' l2 <- ssim_raster(
#'   map1,
#'   map2,
#'   global = FALSE,
#'   w = 1,
#'   do_test = TRUE,
#'   local_test = TRUE,
#'   R = 999
#' )
#' }
#'
#' @importFrom terra global focal crop mask compareGeom values nlyr ifel
#' @export
ssim_raster <- function(map1,
                        map2,
                        global = TRUE,
                        w = 1,
                        transform = c("normal_score", "percentile", "none", "minmax"),
                        k1 = NULL,
                        k2 = NULL,
                        do_test = FALSE,
                        local_test = FALSE,
                        R = 1000,
                        fdr = TRUE,
                        alpha = 0.05,
                        seed = NULL) {

  transform <- match.arg(transform)

  if (!inherits(map1, "SpatRaster") || !inherits(map2, "SpatRaster")) {
    stop("map1 and map2 must be terra SpatRaster objects.")
  }

  if (terra::nlyr(map1) != 1 || terra::nlyr(map2) != 1) {
    stop("map1 and map2 must each have exactly one layer.")
  }

  if (!terra::compareGeom(map1, map2, stopOnError = FALSE)) {
    stop("map1 and map2 must have the same extent, resolution, CRS, and geometry.")
  }

  if (!is.logical(global) || length(global) != 1) {
    stop("global must be TRUE or FALSE.")
  }

  if (!is.logical(do_test) || length(do_test) != 1) {
    stop("do_test must be TRUE or FALSE.")
  }

  if (!is.logical(local_test) || length(local_test) != 1) {
    stop("local_test must be TRUE or FALSE.")
  }

  if (w < 1 || w != floor(w)) {
    stop("w must be a positive integer. w = 1 means a 3x3 window.")
  }

  if (do_test && (R < 1 || R != floor(R))) {
    stop("R must be a positive integer.")
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }

  map1 <- map1[[1]]
  map2 <- map2[[1]]

  # -------------------------------------------------------------------
  # Helper functions
  # -------------------------------------------------------------------

  .minmax <- function(x) {
    mn <- terra::global(x, "min", na.rm = TRUE)[1, 1]
    mx <- terra::global(x, "max", na.rm = TRUE)[1, 1]
    c(min = mn, max = mx)
  }

  .rank_transform <- function(x, type = c("percentile", "normal_score")) {
    type <- match.arg(type)

    vals <- terra::values(x, mat = FALSE)
    idx <- which(!is.na(vals))

    out <- rep(NA_real_, length(vals))

    if (length(idx) == 0) {
      y <- x
      terra::values(y) <- out
      return(y)
    }

    rnk <- rank(vals[idx], ties.method = "average")
    p <- (rnk - 0.5) / length(rnk)

    if (type == "percentile") {
      out[idx] <- p
    }

    if (type == "normal_score") {
      out[idx] <- stats::qnorm(p)
    }

    y <- x
    terra::values(y) <- out
    y
  }

  .scale01 <- function(x) {
    mm <- .minmax(x)

    if (!is.finite(mm["min"]) || !is.finite(mm["max"])) {
      stop("Raster has no finite values.")
    }

    if (mm["max"] == mm["min"]) {
      return(x * 0)
    }

    (x - mm["min"]) / (mm["max"] - mm["min"])
  }

  .apply_transform <- function(x, transform) {
    if (transform == "none") {
      return(x)
    }

    if (transform == "minmax") {
      return(.scale01(x))
    }

    if (transform == "percentile") {
      return(.rank_transform(x, "percentile"))
    }

    if (transform == "normal_score") {
      return(.rank_transform(x, "normal_score"))
    }
  }

  .ssim_layers <- function(x, y, w, k1, k2) {
    if (is.null(k1)) k1 <- 0.01
    if (is.null(k2)) k2 <- 0.03

    if (k1 <= 0 || k1 >= 1) {
      stop("k1 must be greater than 0 and smaller than 1.")
    }

    if (k2 <= 0 || k2 >= 1) {
      stop("k2 must be greater than 0 and smaller than 1.")
    }

    mm_x <- .minmax(x)
    mm_y <- .minmax(y)

    data_min <- min(mm_x["min"], mm_y["min"], na.rm = TRUE)
    data_max <- max(mm_x["max"], mm_y["max"], na.rm = TRUE)
    L_range <- data_max - data_min

    if (!is.finite(L_range) || L_range == 0) {
      L_range <- 1
    }

    C1 <- (k1 * L_range)^2
    C2 <- (k2 * L_range)^2
    C3 <- C2 / 2

    win_size <- (2 * w) + 1

    kernel <- matrix(
      1 / (win_size^2),
      nrow = win_size,
      ncol = win_size
    )

    fmean <- function(z) {
      terra::focal(
        z,
        w = kernel,
        fun = "sum",
        na.policy = "all",
        fillvalue = NA
      )
    }

    mu1 <- fmean(x)
    mu2 <- fmean(y)

    sig1 <- sqrt(abs(fmean(x * x) - mu1 * mu1))
    sig2 <- sqrt(abs(fmean(y * y) - mu2 * mu2))
    sig12 <- fmean(x * y) - mu1 * mu2

    SIM <- ((2 * mu1 * mu2) + C1) / ((mu1^2) + (mu2^2) + C1)
    SIV <- ((2 * sig1 * sig2) + C2) / ((sig1^2) + (sig2^2) + C2)
    SIP <- (sig12 + C3) / ((sig1 * sig2) + C3)

    SSIM <- SIM * SIV * SIP

    out <- c(SSIM, SIM, SIV, SIP)
    names(out) <- c("SSIM", "SIM", "SIV", "SIP")
    out
  }

  .summary_table <- function(x) {
    data.frame(
      metric = names(x),
      mean = as.numeric(terra::global(x, "mean", na.rm = TRUE)[, 1]),
      min  = as.numeric(terra::global(x, "min",  na.rm = TRUE)[, 1]),
      max  = as.numeric(terra::global(x, "max",  na.rm = TRUE)[, 1]),
      sd   = as.numeric(terra::global(x, stats::sd, na.rm = TRUE)[, 1]),
      row.names = NULL
    )
  }

  .permute_raster <- function(x, valid_idx) {
    vals <- terra::values(x, mat = FALSE)

    out <- rep(NA_real_, length(vals))
    out[valid_idx] <- sample(vals[valid_idx], length(valid_idx), replace = FALSE)

    y <- x
    terra::values(y) <- out
    y
  }

  .p_adjust_raster <- function(p_raster) {
    q_raster <- p_raster

    for (i in seq_len(terra::nlyr(p_raster))) {
      vals <- terra::values(p_raster[[i]], mat = FALSE)
      idx <- which(!is.na(vals))

      q_vals <- rep(NA_real_, length(vals))
      q_vals[idx] <- stats::p.adjust(vals[idx], method = "BH")

      terra::values(q_raster[[i]]) <- q_vals
    }

    names(q_raster) <- sub("_p$", "_q", names(p_raster))
    q_raster
  }

  .sig_raster <- function(test_raster, alpha) {
    sig <- test_raster <= alpha
    sig <- terra::ifel(is.na(sig), NA, sig)
    names(sig) <- sub("_(p|q)$", "_sig", names(test_raster))
    sig
  }

  # -------------------------------------------------------------------
  # Pre-processing
  # -------------------------------------------------------------------

  valid <- !is.na(map1) & !is.na(map2)

  map1 <- terra::mask(map1, valid, maskvalues = FALSE)
  map2 <- terra::mask(map2, valid, maskvalues = FALSE)

  map1 <- .apply_transform(map1, transform)
  map2 <- .apply_transform(map2, transform)

  valid_vals <- terra::values(valid, mat = FALSE)
  valid_idx <- which(!is.na(valid_vals) & valid_vals == 1)

  if (length(valid_idx) == 0) {
    stop("There are no overlapping non-NA cells between map1 and map2.")
  }

  # -------------------------------------------------------------------
  # Observed SSIM
  # -------------------------------------------------------------------

  observed <- .ssim_layers(map1, map2, w, k1, k2)
  observed <- terra::mask(observed, valid, maskvalues = FALSE)

  obs_summary <- .summary_table(observed)

  # -------------------------------------------------------------------
  # Case 1: No statistical test
  # -------------------------------------------------------------------

  if (!do_test) {
    if (global) {
      return(list(
        summary = obs_summary,
        settings = list(
          global = global,
          w = w,
          transform = transform,
          do_test = do_test,
          local_test = local_test
        )
      ))
    }

    return(observed)
  }

  # -------------------------------------------------------------------
  # Case 2: global = TRUE, do_test = TRUE
  # Global permutation test
  # -------------------------------------------------------------------

  if (global) {
    if (local_test) {
      warning("local_test = TRUE is ignored because global = TRUE.")
    }

    null_means <- matrix(
      NA_real_,
      nrow = R,
      ncol = terra::nlyr(observed)
    )

    colnames(null_means) <- names(observed)

    for (i in seq_len(R)) {
      map2_perm <- .permute_raster(map2, valid_idx)
      perm_layers <- .ssim_layers(map1, map2_perm, w, k1, k2)
      perm_layers <- terra::mask(perm_layers, valid, maskvalues = FALSE)

      null_means[i, ] <- as.numeric(
        terra::global(perm_layers, "mean", na.rm = TRUE)[, 1]
      )
    }

    obs_means <- obs_summary$mean

    p_values <- vapply(seq_along(obs_means), function(j) {
      (sum(null_means[, j] >= obs_means[j], na.rm = TRUE) + 1) / (R + 1)
    }, numeric(1))

    obs_summary$p_value <- p_values

    if (isTRUE(fdr)) {
      obs_summary$q_value <- stats::p.adjust(obs_summary$p_value, method = "BH")
      obs_summary$significant <- obs_summary$q_value <= alpha
    } else {
      obs_summary$significant <- obs_summary$p_value <= alpha
    }

    return(list(
      summary = obs_summary,
      null_distribution = as.data.frame(null_means),
      settings = list(
        global = global,
        w = w,
        transform = transform,
        do_test = do_test,
        local_test = local_test,
        R = R,
        fdr = fdr,
        alpha = alpha,
        seed = seed
      )
    ))
  }

  # -------------------------------------------------------------------
  # Case 3: global = FALSE, do_test = TRUE, local_test = FALSE
  # Return local SSIM only, no local p-value
  # -------------------------------------------------------------------

  if (!global && do_test && !local_test) {
    warning(
      "do_test = TRUE but local_test = FALSE. ",
      "Returning local SSIM layers only. ",
      "Use local_test = TRUE for local p-value/q-value layers."
    )

    return(observed)
  }

  # -------------------------------------------------------------------
  # Case 4: global = FALSE, do_test = TRUE, local_test = TRUE
  # Local permutation test
  # -------------------------------------------------------------------

  count_ge <- observed * 0
  count_ge <- terra::ifel(is.na(count_ge), 0, count_ge)

  for (i in seq_len(R)) {
    map2_perm <- .permute_raster(map2, valid_idx)
    perm_layers <- .ssim_layers(map1, map2_perm, w, k1, k2)
    perm_layers <- terra::mask(perm_layers, valid, maskvalues = FALSE)

    ge <- perm_layers >= observed
    ge <- terra::ifel(is.na(ge), 0, ge)

    count_ge <- count_ge + ge
  }

  p_layers <- (count_ge + 1) / (R + 1)
  p_layers <- terra::mask(p_layers, observed[[1]])
  names(p_layers) <- paste0(names(observed), "_p")

  if (isTRUE(fdr)) {
    q_layers <- .p_adjust_raster(p_layers)
    sig_layers <- .sig_raster(q_layers, alpha)

    out <- c(observed, p_layers, q_layers, sig_layers)
  } else {
    sig_layers <- .sig_raster(p_layers, alpha)

    out <- c(observed, p_layers, sig_layers)
  }

  return(out)
}

#' Toronto neighbourhood deprivation indices and commuting pattern (2016)
#'
#' An \code{sf} polygon object for the Toronto Census Subdivision, Ontario,
#' Canada. Each polygon corresponds to a Census Tract (CT) and includes two
#' neighbourhood-level deprivation indices and one census variable, which can
#' be used as inputs to \code{\link{ssim_polygon}} and
#' \code{\link{ssim_bandwidth}}.
#'
#' @format An \code{sf} data frame with one row per Census Tract and the
#'   following key columns:
#'   \describe{
#'     \item{CIMD}{Canadian Index of Multiple Deprivation (situational
#'       dimension), 2016. Higher values indicate greater deprivation.}
#'     \item{PP}{Pampalon deprivation index (social dimension), 2016.}
#'     \item{Commute}{Percentage of households commuting within the
#'       Census Subdivision of residence, 2016.}
#'     \item{geometry}{\code{sfc} polygon geometry column.}
#'   }
#'
#' @source
#'   Statistics Canada, 2016 Census of Population.
#'
#' @examples
#' data("Toronto_SSIM")
#' plot(Toronto_SSIM["CIMD"], border = NA)
"Toronto_SSIM"


#' British Columbia Fire Weather Index — 16 August 2023
#'
#' Daily Canadian Fire Weather Index (FWI) for the province of British
#' Columbia, Canada, on 16 August 2023, at a 2 km spatial resolution. This
#' date represents conditions during the peak summer fire season, when fire
#' danger levels in British Columbia are typically high.
#'
#' The object is stored as a packed \code{terra} raster (created with
#' \code{terra::wrap()}) so that it can be saved in an R \code{.rda} file.
#' Use \code{terra::unwrap()} to convert it back to a regular \code{terra}
#' raster before passing it to \code{\link{ssim_raster}} or to other
#' \code{terra} functions.
#'
#' @format A packed \code{terra} raster object with one numeric layer
#'   (FWI values).
#'
#' @source
#'   Canadian Forest Fire Weather Index System; FWI values are derived from
#'   meteorological inputs (temperature, wind speed, relative humidity,
#'   precipitation).
#'
#' @examples
#' data("fwi_0816_bc")
#' fwi_0816_bc <- terra::unwrap(fwi_0816_bc)
#' terra::plot(fwi_0816_bc, main = "FWI - 16 August 2023")
"fwi_0816_bc"


#' British Columbia Fire Weather Index — 18 August 2023
#'
#' Daily Canadian Fire Weather Index (FWI) for the province of British
#' Columbia, Canada, on 18 August 2023, at a 2 km spatial resolution. This
#' date also represents conditions during the peak summer fire season and is
#' typically expected to show high spatial similarity to
#' \code{\link{fwi_0816_bc}}.
#'
#' The object is stored as a packed \code{terra} raster (created with
#' \code{terra::wrap()}). Use \code{terra::unwrap()} to convert it back to a
#' regular \code{terra} raster before use.
#'
#' @format A packed \code{terra} raster object with one numeric layer
#'   (FWI values).
#'
#' @source Canadian Forest Fire Weather Index System.
#'
#' @examples
#' data("fwi_0818_bc")
#' fwi_0818_bc <- terra::unwrap(fwi_0818_bc)
#' terra::plot(fwi_0818_bc, main = "FWI - 18 August 2023")
"fwi_0818_bc"


#' British Columbia Fire Weather Index — 1 November 2023
#'
#' Daily Canadian Fire Weather Index (FWI) for the province of British
#' Columbia, Canada, on 1 November 2023, at a 2 km spatial resolution. This
#' date represents late-fall conditions, when fire danger across most of
#' British Columbia is typically much lower than during the summer season due
#' to cooler temperatures and increased precipitation. The spatial structure
#' of this map is therefore expected to differ substantially from the summer
#' FWI maps (\code{\link{fwi_0816_bc}}, \code{\link{fwi_0818_bc}}).
#'
#' The object is stored as a packed \code{terra} raster (created with
#' \code{terra::wrap()}). Use \code{terra::unwrap()} to convert it back to a
#' regular \code{terra} raster before use.
#'
#' @format A packed \code{terra} raster object with one numeric layer
#'   (FWI values).
#'
#' @source Canadian Forest Fire Weather Index System.
#'
#' @examples
#' data("fwi_1101_bc")
#' fwi_1101_bc <- terra::unwrap(fwi_1101_bc)
#' terra::plot(fwi_1101_bc, main = "FWI - 1 November 2023")
"fwi_1101_bc"

---
title: "Introduction to SSIMmap"
output: 
  rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to SSIMmap}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

The goal of the *SSIMmap* package is to extend [the classical SSIM method](https://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=1284395) to irregular lattice-based maps and raster images. The original SSIM method was developed to compare two images by mimicking the human visual system. The *SSIMmap* package applies this method to two types of maps (polygon and raster). For irregular lattice-based maps, the geographical SSIM method incorporates [geographically weighted summary statistics](https://www.sciencedirect.com/science/article/pii/S0198971501000096?via%3Dihub) with an adaptive k-nearest-neighbour (k-NN) Gaussian kernel.

This package includes three key functions: **`ssim_bandwidth`**, **`ssim_polygon`**, and **`ssim_raster`**. In addition to point estimates of SSIM and its components (SIM, SIV, SIP), the polygon and raster functions also support **permutation tests** for global and local significance, with optional Benjamini–Hochberg FDR correction for local p-values.

Users who want to compare two maps and quantify their similarities can use this package and visualize the results with other R packages (e.g., [*tmap*](https://CRAN.R-project.org/package=tmap), [*ggplot2*](https://CRAN.R-project.org/package=ggplot2), [*patchwork*](https://CRAN.R-project.org/package=patchwork), and [*tidyterra*](https://CRAN.R-project.org/package=tidyterra)).

```{r setup, echo = FALSE}
suppressPackageStartupMessages(library(SSIMmap))
knitr::opts_chunk$set(warning = FALSE, message = FALSE) 
```

## Example data

The package ships with the following example data sets:

1. **`Toronto_SSIM`** — an `sf` polygon object for Toronto, ON, containing two neighbourhood deprivation indices (the Canadian Index of Multiple Deprivation, `CIMD`, and the Pampalon index, `PP`) and a census variable (`Commute`: percentage of households commuting within the census subdivision of residence) for 2016.

2. **Daily Canadian Fire Weather Index (FWI) maps for British Columbia (BC)** — three packed `terra` raster objects at a 2 km spatial resolution, derived from the Canadian Forest Fire Weather Index System. FWI is a numeric rating that is driven by meteorological inputs such as temperature, wind speed, humidity and precipitation, and typically ranges from near 0 (minimal fire danger) to values exceeding 100 (extreme fire-weather conditions).

   The three dates were chosen to represent contrasting fire-weather conditions:
   
   - `fwi_0816_bc` — 16 August 2023 (peak summer fire season)
   - `fwi_0818_bc` — 18 August 2023 (peak summer fire season)
   - `fwi_1101_bc` — 1 November 2023 (late fall)
   
   Two days within the summer season are expected to be highly similar in space, whereas a summer–fall comparison should show much lower spatial similarity, because fall conditions in western Canada are typically cooler and wetter, with reduced fire danger across most of the province.

   Because `terra` raster objects rely on external pointers, the FWI rasters are stored as `PackedSpatRaster` objects (created with `terra::wrap()`) and need to be converted back to `SpatRaster` with `terra::unwrap()` before use.

```{r, message = FALSE}
library(SSIMmap)
library(sf)
library(terra)
library(ggplot2)
library(patchwork)
library(tidyterra)

# Polygon data
data("Toronto_SSIM")
plot(Toronto_SSIM["CIMD"], border = NA, main = "CIMD")

# Raster data (FWI for British Columbia)
# Stored as PackedSpatRaster — unwrap to SpatRaster for use with terra
data("fwi_0816_bc"); fwi_0816_bc <- terra::unwrap(fwi_0816_bc)
data("fwi_0818_bc"); fwi_0818_bc <- terra::unwrap(fwi_0818_bc)
data("fwi_1101_bc"); fwi_1101_bc <- terra::unwrap(fwi_1101_bc)

plot(fwi_0816_bc, main = "FWI – 16 August 2023")
plot(fwi_0818_bc, main = "FWI – 18 August 2023")
plot(fwi_1101_bc, main = "FWI – 1 November 2023")
```

## Function: `ssim_bandwidth`

`ssim_bandwidth()` selects a bandwidth (number of nearest neighbours, *k*) for the adaptive k-NN Gaussian kernel used in `ssim_polygon()`, by computing **bias–variance trade-off curves** for each of the two input variables. Note that this function does *not* compute SSIM itself; it is a helper for choosing a suitable bandwidth.

Key arguments:

- `shape`: an `sf` polygon object.
- `map1`, `map2`: column names in `shape` for the two variables.
- `max_bandwidth`: upper bound of *k* (must be $\geq 12$ and not exceed the number of polygons).
- `transform`: one of `"normal_score"` (Blom normal scores), `"percentile"` (empirical percentiles), `"none"`, or `"minmax"` (min–max normalization to [0, 1]). Use a transformation when the two variables have very different scales or units.
- `option`: how to choose a single bandwidth from the suggested range — `"midpoint"` (default), `"lower"`, or `"upper"`.

The function returns a list with three elements:

- `plot`: a `ggplot` object showing standardized bias (solid line) and variance (dashed line) against bandwidth, with the suggested range highlighted.
- `bandwidth`: the chosen bandwidth `k_star`.
- `tradeoff`: the underlying trade-off data and `sqrt(n)` as a reference.

```{r}
args(ssim_bandwidth)
```

### How to execute

```{r, fig.width = 6, fig.height = 4}
S1 <- ssim_bandwidth(
  shape         = Toronto_SSIM,
  map1          = "CIMD",
  map2          = "PP",
  max_bandwidth = 100,
  transform     = "percentile",
  option        = "midpoint"
)
S1$bandwidth
S1$plot
```

```{r, fig.width = 6, fig.height = 4}
S2 <- ssim_bandwidth(
  shape         = Toronto_SSIM,
  map1          = "CIMD",
  map2          = "Commute",
  max_bandwidth = 100,
  transform     = "percentile",
  option        = "midpoint"
)
S2$bandwidth
S2$plot
```

### Combining two bandwidth plots with `patchwork`

When comparing several pairs of maps, you can place the trade-off plots side-by-side using **`patchwork`**:

```{r, fig.width = 10, fig.height = 5}
# Tag each panel and remove individual captions
p1 <- S1$plot +
  labs(tag = "a", caption = NULL) +
  theme(plot.tag = element_text(face = "bold", size = 14))

p2 <- S2$plot +
  labs(tag = "b", caption = NULL) +
  theme(plot.tag = element_text(face = "bold", size = 14))

# Combine side-by-side with a shared legend at the bottom
combined_bandwidth_plot <- (p1 + p2) +
  plot_layout(guides = "collect") &
  theme(legend.position = "bottom")

# Add a single global caption
combined_bandwidth_plot +
  plot_annotation(
    caption = "* Solid line: Bias / Dashed line: Variance",
    theme   = theme(plot.caption = element_text(size = 10, hjust = 0.5))
  )
```

## Function: `ssim_polygon`

`ssim_polygon()` computes SSIM and its components (`SIM`, `SIV`, `SIP`) for two numeric variables stored in a polygon `sf` object. Local statistics are calculated using an adaptive k-nearest-neighbour Gaussian kernel based on polygon centroids. The number of neighbours is controlled by `bandwidth`; if `bandwidth = NULL`, the default is `ceiling(sqrt(n))`, where `n` is the number of polygons.

Depending on the arguments, the function can return:

- a **global summary table** for `SSIM`, `SIM`, `SIV`, and `SIP`
  when `global = TRUE`, or
- an **`sf` object with local `SSIM`, `SIM`, `SIV`, and `SIP` values**
  for each polygon when `global = FALSE`.

The argument `transform` controls preprocessing of the two input variables. Available options are `"normal_score"`, `"percentile"`, `"minmax"`, and `"none"`.

When `do_test = TRUE`, the function performs permutation-based inference:

- **Global test** (`global = TRUE`): returns two-sided permutation p-values
  for global `SSIM`, `SIM`, `SIV`, and `SIP`.
- **Local test** (`global = FALSE`): returns per-polygon SSIM p-values in
  `p_value`. If `fdr = TRUE`, Benjamini-Hochberg FDR-adjusted values are
  returned in `q_value`; otherwise, `q_value` is equal to `p_value`. The logical
  column `sig` flags polygons with `q_value < alpha`.

Under the null hypothesis, both variables are randomly permuted at each iteration, breaking both spatial structure and association between the two,variables.

```{r}
args(ssim_polygon)
```

### Global SSIM with permutation test

```{r}
S1_global <- ssim_polygon(
  shape     = Toronto_SSIM,
  map1      = "CIMD",
  map2      = "PP",
  global    = TRUE,
  bandwidth = 87,
  transform = "percentile",
  k1 = NULL, k2 = NULL,
  do_test = TRUE, R = 1000, fdr = TRUE, alpha = 0.05,
  seed = 1
)

# Global means and permutation p-values
S1_global$p_global
```

```{r}
S2_global <- ssim_polygon(
  shape     = Toronto_SSIM,
  map1      = "CIMD",
  map2      = "Commute",
  global    = TRUE,
  bandwidth = 91,
  transform = "percentile",
  do_test = TRUE, R = 1000, fdr = TRUE, alpha = 0.05,
  seed = 1
)
S2_global$p_global
```

### Local SSIM with permutation test and FDR correction

When `global = FALSE`, the function returns an `sf` object that can be passed directly to mapping packages such as `ggplot2` or `tmap`.

```{r}
S1_local <- ssim_polygon(
  shape     = Toronto_SSIM,
  map1      = "CIMD",
  map2      = "PP",
  global    = FALSE,
  bandwidth = 87,
  transform = "percentile",
  do_test = TRUE, R = 1000, fdr = TRUE, alpha = 0.05,
  seed = 1
)
head(S1_local[, c("SSIM", "SIM", "SIV", "SIP", "p_value", "q_value", "sig")])
```

```{r}
S2_local <- ssim_polygon(
  shape     = Toronto_SSIM,
  map1      = "CIMD",
  map2      = "Commute",
  global    = FALSE,
  bandwidth = 91,
  transform = "percentile",
  do_test = TRUE, R = 1000, fdr = TRUE, alpha = 0.05,
  seed = 1
)
```

### Visualizing local SSIM with `ggplot2`

```{r, fig.show = "hold", out.width = "45%"}
library(RColorBrewer)

# CIMD vs. Pampalon
ggplot() +
  geom_sf(data = S1_local, aes(fill = SSIM), color = NA) +
  scale_fill_gradientn(colors = brewer.pal(8, "PRGn"), limits = c(-1, 1)) +
  theme_void()

# CIMD vs. Commute
ggplot() +
  geom_sf(data = S2_local, aes(fill = SSIM), color = NA) +
  scale_fill_gradientn(colors = brewer.pal(8, "PRGn"), limits = c(-1, 1)) +
  theme_void()
```

You can also overlay FDR-significant polygons (`sig = TRUE`) on top of the local SSIM map by adding a separate layer with `geom_sf(data = subset(S1_local, sig), ...)`.

## Function: `ssim_raster`

`ssim_raster()` computes the Structural Similarity Index Measure (SSIM) between two raster images. It also returns the three SSIM components: `SIM`, `SIV`, and `SIP`.

The function uses a moving window of size $(2w + 1) \times (2w + 1)$.For example, the default `w = 1` uses a `3 x 3` window. Inputs must be single-layer `terra::SpatRaster` objects with the same geometry.

- If `global = TRUE`, the function returns a list containing a global summary table for `SSIM`, `SIM`, `SIV`, and `SIP`.
- If `global = FALSE`, the function returns a multi-layer `terra::SpatRaster` with per-cell `SSIM`, `SIM`, `SIV`, and `SIP`.

The argument `transform` controls the preprocessing applied to both raster maps before SSIM calculation. Available options are `"normal_score"`,
`"percentile"`, `"minmax"`, and `"none"`.

When `do_test = TRUE`, permutation-based statistical inference is performed.

- If `global = TRUE`, the function returns global p-values and, when
  `fdr = TRUE`, FDR-adjusted q-values.
- If `global = FALSE` and `local_test = TRUE`, the function returns local
  p-value layers (`SSIM_p`, `SIM_p`, `SIV_p`, `SIP_p`), FDR-adjusted q-value
  layers (`SSIM_q`, `SIM_q`, `SIV_q`, `SIP_q`) when `fdr = TRUE`, and
  significance layers (`SSIM_sig`, `SIM_sig`, `SIV_sig`, `SIP_sig`).

```{r}
args(ssim_raster)
```

### Global SSIM

A quick global comparison between the two summer FWI maps (16 vs. 18 August 2023) and between summer and fall (16 August vs. 1 November 2023):

```{r}
ssim_raster(fwi_0816_bc, fwi_0818_bc)   # summer vs summer (high similarity expected)
ssim_raster(fwi_0816_bc, fwi_1101_bc)   # summer vs fall   (lower similarity expected)
```

### Local SSIM with a permutation test

In the example below we use `w = 1` (a $3 \times 3$ moving window), `do_test = TRUE` and `local_test = TRUE` to obtain both the local SSIM surfaces and per-cell p-values. The chunks are set to `eval = FALSE` to keep the vignette build time short — when running interactively you can simply remove that option.

```{r, eval = FALSE}
# Comparison 1: summer vs summer
p1_l <- ssim_raster(
  fwi_0816_bc, fwi_0818_bc,
  global     = FALSE,
  w          = 1,
  do_test    = TRUE,
  local_test = TRUE,
    fdr= TRUE,
  R          = 999
)

# Comparison 2: summer vs fall
p2_l <- ssim_raster(
  fwi_0816_bc, fwi_1101_bc,
  global     = FALSE,
  w          = 1,
  do_test    = TRUE,
  local_test = TRUE,
  fdr= TRUE,
  R          = 999
)
```

### Visualizing the SSIM components with `tidyterra`

We re-project the SSIM result to an Albers Equal-Area projection (EPSG:3005, the standard projection for Statistics Canada / British Columbia) for area-faithful display, then use `tidyterra::geom_spatraster()` to facet over the four SSIM layers:

```{r, eval = FALSE}
crs_albers <- "EPSG:3005"

# Re-project local SSIM result (continuous values → bilinear)
p1_l_alb <- terra::project(p1_l, crs_albers, method = "bilinear")

# Select SSIM + components and assign panel labels
ssim_maps_alb <- p1_l_alb[[c("SSIM", "SIM", "SIV", "SIP")]]
facet_labs <- c(SSIM = "a", SIM = "b", SIV = "c", SIP = "d")

gg <- ggplot() +
  geom_spatraster(data = ssim_maps_alb) +
  facet_wrap(~lyr, ncol = 2, labeller = labeller(lyr = facet_labs)) +
  scale_fill_viridis_c(
    limits   = c(-1, 1),
    na.value = "transparent",
    name     = "Values"
  ) +
  coord_sf(crs = crs_albers, expand = FALSE) +
  theme_minimal() +
  theme(
    strip.text       = element_text(size = 14, face = "bold", hjust = 0),
    strip.background = element_blank(),
    legend.title     = element_text(size = 13, face = "bold"),
    legend.text      = element_text(size = 11),
    panel.background = element_blank(),
    panel.grid       = element_blank()
  )
gg
```

### Comparing two SSIM rasters side-by-side with `patchwork`

A common workflow is to compare a baseline image against several follow-up images and place the resulting SSIM maps in a single figure. Below we build one panel for each comparison (summer vs summer; summer vs fall) and combine them with a shared legend:

```{r, eval = FALSE}
# Re-project the second comparison and keep only SSIM
p2_l_alb         <- terra::project(p2_l, crs_albers, method = "bilinear")
ssim_maps_alb_p2 <- p2_l_alb[[c("SSIM")]]

ssim_maps_alb_p2 <- p2_l_alb[[c("SSIM", "SIM", "SIV", "SIP")]]
facet_labs <- c(SSIM = "a", SIM = "b", SIV = "c", SIP = "d")

gg2 <- ggplot() +
  geom_spatraster(data = ssim_maps_alb_p2) +
  facet_wrap(~lyr, ncol = 2, labeller = labeller(lyr = facet_labs)) +
  scale_fill_viridis_c(
    limits   = c(-1, 1),
    na.value = "transparent",
    name     = "Values"
  ) +
  coord_sf(crs = crs_albers, expand = FALSE) +
  theme_minimal() +
  theme(
    strip.text       = element_text(size = 14, face = "bold", hjust = 0),
    strip.background = element_blank(),
    legend.title     = element_text(size = 13, face = "bold"),
    legend.text      = element_text(size = 11),
    panel.background = element_blank(),
    panel.grid       = element_blank())
gg2
```

We expect the summer–summer comparison (panel **a**) to show high SSIM values across most of British Columbia, whereas the summer–fall comparison (panel **b**) should show substantially lower similarity, reflecting the seasonal shift from dry summer fire weather to cooler, wetter fall conditions.

### Per-cell p-value maps

When `do_test = TRUE` and `local_test = TRUE`, the returned `SpatRaster` also contains `SSIM_p`, `SIM_p`, `SIV_p`, and `SIV_p` layers, which can be plotted directly:

```{r, eval = FALSE}
p_maps <- p1_l[[c("SSIM_p", "SIM_p", "SIV_p", "SIV_p")]]
plot(p_maps,
     main = c("p-value: SSIM", "p-value: SIM",
              "p-value: SIV",  "p-value: SIP"))
```

## References

* Brunsdon, C., Fotheringham, A.S. and Charlton, M. (2002). Geographically weighted summary statistics — a framework for localised exploratory data analysis. *Computers, Environment and Urban Systems*, 26(6), pp. 501–524. <https://doi.org/10.1016/S0198-9715(01)00009-6>
* Dimitrakopoulos, A.P., Bemmerzouk, A.M. and Mitsopoulos, I.D. (2011). Evaluation of the Canadian fire weather index system in an eastern Mediterranean environment. *Meteorological Applications*, 18(1), pp. 83–93. <https://doi.org/10.1002/met.214>
* Lu, B., Harris, P., Charlton, M. and Brunsdon, C. (2014). The GWmodel R package: further topics for exploring spatial heterogeneity using geographically weighted models. *Geo-spatial Information Science*, 17(2), pp. 85–101. <https://doi.org/10.1080/10095020.2014.917453>
* Wang, Z., Bovik, A.C., Sheikh, H.R. and Simoncelli, E.P. (2004). Image quality assessment: from error visibility to structural similarity. *IEEE Transactions on Image Processing*, 13(4), pp. 600–612. <https://doi.org/10.1109/TIP.2003.819861>
* Benjamini, Y. and Hochberg, Y. (1995). Controlling the false discovery rate: a practical and powerful approach to multiple testing. *Journal of the Royal Statistical Society: Series B*, 57(1), pp. 289–300. <https://doi.org/10.1111/j.2517-6161.1995.tb02031.x>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Toronto_SSIM}
\alias{Toronto_SSIM}
\title{Toronto neighbourhood deprivation indices and commuting pattern (2016)}
\format{
An \code{sf} data frame with one row per Census Tract and the
  following key columns:
  \describe{
    \item{CIMD}{Canadian Index of Multiple Deprivation (situational
      dimension), 2016. Higher values indicate greater deprivation.}
    \item{PP}{Pampalon deprivation index (social dimension), 2016.}
    \item{Commute}{Percentage of households commuting within the
      Census Subdivision of residence, 2016.}
    \item{geometry}{\code{sfc} polygon geometry column.}
  }
}
\source{
Statistics Canada, 2016 Census of Population.
}
\usage{
Toronto_SSIM
}
\description{
An \code{sf} polygon object for the Toronto Census Subdivision, Ontario,
Canada. Each polygon corresponds to a Census Tract (CT) and includes two
neighbourhood-level deprivation indices and one census variable, which can
be used as inputs to \code{\link{ssim_polygon}} and
\code{\link{ssim_bandwidth}}.
}
\examples{
data("Toronto_SSIM")
plot(Toronto_SSIM["CIMD"], border = NA)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssim_bandwith.R
\name{ssim_bandwidth}
\alias{ssim_bandwidth}
\title{Bandwidth selection for polygon maps}
\usage{
ssim_bandwidth(
  shape,
  map1,
  map2,
  max_bandwidth,
  transform = c("normal_score", "percentile", "none", "minmax"),
  option = "midpoint"
)
}
\arguments{
\item{shape}{An \code{sf} polygon object with attributes containing the two
variables of interest.}

\item{map1}{Character string; column name in \code{shape} for the first map.}

\item{map2}{Character string; column name in \code{shape} for the second map.}

\item{max_bandwidth}{Integer (at least 12). Upper bound of the bandwidth
\code{k} for the adaptive k-nearest-neighbour kernel. Must not exceed the
number of polygons.}

\item{transform}{One of \code{c("normal_score","percentile","none","minmax")}.
\code{"normal_score"} applies a Blom normal scores transform;
\code{"percentile"} maps values to empirical percentiles in (0, 1);
\code{"minmax"} applies min–max normalisation to [0, 1]; and
\code{"none"} leaves the variables on their original scale.}

\item{option}{Character string specifying how to choose a single bandwidth
from the bias–variance trade-off range: one of
\code{"midpoint"}, \code{"lower"}, or \code{"upper"} (default
\code{"midpoint"}). The square-root of the number of polygons (\code{sqrt(n)})
is also shown on the plot as a reference.}
}
\value{
A list with components:
  \item{plot}{A \code{ggplot} object showing the bias–variance trade-off
    curves for both variables, the suggested bandwidth range, the chosen
    bandwidth, and \code{sqrt(n)} as a reference.}
  \item{bandwidth}{The chosen bandwidth \code{k_star}.}
  \item{tradeoff}{A list with elements
    \code{map1}, \code{map2}, \code{kA}, \code{kB}, and \code{k_sqrt},
    where \code{map1} and \code{map2} are data frames containing the
    standardized bias and variance across \code{k} for each variable.}
}
\description{
Computes bias–variance trade-off curves for two polygon maps using an
adaptive Gaussian kernel. The two input variables can be optionally
transformed (e.g. normal scores or min–max normalisation) before smoothing.
A suggested bandwidth range is obtained by balancing standardized bias and
variance for each variable, and a single bandwidth is chosen from this range.
}
\details{
This function does *not* compute SSIM; it is intended to be used in
combination with \code{ssim_polygon()}, which can then be called with the
selected bandwidth.


The kernel is Gaussian with adaptive bandwidth:
\deqn{w_{ij} = \exp\left\{-\tfrac12 (d_{ij} / h_i)^2 \right\},}
where \eqn{d_{ij}} is the distance between polygon centroids and
\eqn{h_i} is the distance from polygon \eqn{i} to its \eqn{k}-th nearest
neighbour. Bandwidth selection is based purely on bias–variance trade-off
and does not involve any SSIM computation or permutation testing.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fwi_1101_bc}
\alias{fwi_1101_bc}
\title{British Columbia Fire Weather Index — 1 November 2023}
\format{
A packed \code{terra} raster object with one numeric layer
  (FWI values).
}
\source{
Canadian Forest Fire Weather Index System.
}
\usage{
fwi_1101_bc
}
\description{
Daily Canadian Fire Weather Index (FWI) for the province of British
Columbia, Canada, on 1 November 2023, at a 2 km spatial resolution. This
date represents late-fall conditions, when fire danger across most of
British Columbia is typically much lower than during the summer season due
to cooler temperatures and increased precipitation. The spatial structure
of this map is therefore expected to differ substantially from the summer
FWI maps (\code{\link{fwi_0816_bc}}, \code{\link{fwi_0818_bc}}).
}
\details{
The object is stored as a packed \code{terra} raster (created with
\code{terra::wrap()}). Use \code{terra::unwrap()} to convert it back to a
regular \code{terra} raster before use.
}
\examples{
data("fwi_1101_bc")
fwi_1101_bc <- terra::unwrap(fwi_1101_bc)
terra::plot(fwi_1101_bc, main = "FWI - 1 November 2023")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssim_polygon.R
\name{ssim_polygon}
\alias{ssim_polygon}
\title{Structural similarity index (SSIM) for polygon maps}
\usage{
ssim_polygon(
  shape,
  map1,
  map2,
  global = TRUE,
  bandwidth = NULL,
  transform = c("normal_score", "percentile", "none", "minmax"),
  k1 = NULL,
  k2 = NULL,
  do_test = FALSE,
  R = 1000,
  fdr = TRUE,
  alpha = 0.05,
  seed = NULL
)
}
\arguments{
\item{shape}{An \code{sf} polygon object.}

\item{map1}{Character string; column name in \code{shape} for the first map.}

\item{map2}{Character string; column name in \code{shape} for the second map.}

\item{global}{Logical. If \code{TRUE}, compute and print a summary table of
global SSIM, SIM, SIV, SIP. If \code{FALSE}, return an \code{sf} object
with local metrics (and local p-/q-values if \code{do_test = TRUE}).}

\item{bandwidth}{Integer; adaptive k-NN size (number of neighbours). The
default is \code{ceiling(sqrt(n))}, where \code{n} is the number of
polygons. Must be at least 3 and not exceed \code{n}.}

\item{transform}{One of \code{c("normal_score", "percentile", "none", "minmax")}.
\code{"normal_score"} applies a Blom normal scores transform;
\code{"percentile"} maps values to empirical percentiles in (0, 1);
\code{"minmax"} applies min–max normalisation to [0, 1]; and
\code{"none"} leaves the variables on their original scale.}

\item{k1, k2}{SSIM constants. If \code{NULL}, defaults are \code{k1 = 0.01},
\code{k2 = 0.03}.}

\item{do_test}{Logical; if \code{TRUE}, perform permutation tests for global
and local SSIM. Local p-values can be FDR-adjusted if \code{fdr = TRUE}.}

\item{R}{Integer; number of permutations for the significance tests
(default \code{R = 1000}).}

\item{fdr}{Logical; if \code{TRUE} (default), apply Benjamini–Hochberg
FDR correction to local p-values.}

\item{alpha}{Numeric; significance threshold for local results (default
\code{0.05}). Local features with \code{q_value < alpha} are flagged as
significant.}

\item{seed}{Optional integer; random seed for reproducibility of the
permutation tests.}
}
\value{
If \code{global = TRUE} and \code{do_test = FALSE}, the function prints a
  knitr table summarising the global mean, minimum, maximum, and standard
  deviation of SSIM, SIM, SIV, and SIP, and returns this \code{data.frame}
  (invisibly).

  If \code{global = TRUE} and \code{do_test = TRUE}, the function prints the
  same summary table plus a table of global permutation p-values (two-sided)
  for SSIM, SIM, SIV, and SIP, and returns (invisibly) a \code{list} with
  components:
  \itemize{
    \item \code{summary}: \code{data.frame} with global SSIM/SIM/SIV/SIP
      summary statistics;
    \item \code{p_global}: \code{data.frame} with global means and
      permutation p-values for SSIM, SIM, SIV, SIP.
  }

  If \code{global = FALSE}, the function returns an \code{sf} object equal to
  \code{shape} with additional columns:
  \itemize{
    \item \code{SSIM}, \code{SIM}, \code{SIV}, \code{SIP}: local similarity
      metrics for each polygon;
    \item \code{p_value}, \code{q_value}, \code{sig} (only if
      \code{do_test = TRUE}): local permutation p-value, FDR-adjusted q-value,
      and a logical flag indicating significance \code{(q_value < alpha)}.
  }
}
\description{
Computes local and global SSIM (and its components SIM, SIV, SIP) for two
polygon maps using an adaptive k-nearest-neighbour (k-NN) Gaussian kernel.
Input variables can be optionally transformed (e.g. rank-based inverse
normal scores or min–max normalization). 
Optionally performs permutation tests for global and local significance with BH-FDR correction.
Under the null hypothesis for the permutation test, both variables are
randomly permuted at each iteration, breaking spatial structure and
association between them. Local p-values can be optionally adjusted using
the Benjamini–Hochberg FDR procedure.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fwi_0816_bc}
\alias{fwi_0816_bc}
\title{British Columbia Fire Weather Index — 16 August 2023}
\format{
A packed \code{terra} raster object with one numeric layer
  (FWI values).
}
\source{
Canadian Forest Fire Weather Index System; FWI values are derived from
  meteorological inputs (temperature, wind speed, relative humidity,
  precipitation).
}
\usage{
fwi_0816_bc
}
\description{
Daily Canadian Fire Weather Index (FWI) for the province of British
Columbia, Canada, on 16 August 2023, at a 2 km spatial resolution. This
date represents conditions during the peak summer fire season, when fire
danger levels in British Columbia are typically high.
}
\details{
The object is stored as a packed \code{terra} raster (created with
\code{terra::wrap()}) so that it can be saved in an R \code{.rda} file.
Use \code{terra::unwrap()} to convert it back to a regular \code{terra}
raster before passing it to \code{\link{ssim_raster}} or to other
\code{terra} functions.
}
\examples{
data("fwi_0816_bc")
fwi_0816_bc <- terra::unwrap(fwi_0816_bc)
terra::plot(fwi_0816_bc, main = "FWI - 16 August 2023")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssim_raster.R
\name{ssim_raster}
\alias{ssim_raster}
\title{SSIM index for raster images}
\usage{
ssim_raster(
  map1,
  map2,
  global = TRUE,
  w = 1,
  transform = c("normal_score", "percentile", "none", "minmax"),
  k1 = NULL,
  k2 = NULL,
  do_test = FALSE,
  local_test = FALSE,
  R = 1000,
  fdr = TRUE,
  alpha = 0.05,
  seed = NULL
)
}
\arguments{
\item{map1}{A single-layer \code{terra::SpatRaster} object representing the first raster map.}

\item{map2}{A single-layer \code{terra::SpatRaster} object representing the second raster map.}

\item{global}{Logical. If \code{TRUE}, the function returns global summary
statistics of SSIM, SIM, SIV, and SIP. If \code{FALSE}, it returns local
raster layers for each cell. Default is \code{TRUE}.}

\item{w}{Integer. Radius of the local moving window used to calculate local
SSIM components. For example, \code{w = 1} uses a 3 x 3 window, and
\code{w = 2} uses a 5 x 5 window. Default is \code{1}.}

\item{transform}{Character. Transformation applied to both raster maps before
SSIM calculation. Options are \code{"normal_score"}, \code{"percentile"},
\code{"none"}, and \code{"minmax"}. Default is \code{"normal_score"}.}

\item{k1}{Numeric. Constant used to stabilize the luminance component of SSIM.
If \code{NULL}, the default value \code{0.01} is used.}

\item{k2}{Numeric. Constant used to stabilize the contrast and structure
components of SSIM. If \code{NULL}, the default value \code{0.03} is used.}

\item{do_test}{Logical. If \code{TRUE}, a permutation-based statistical test
is performed. Default is \code{FALSE}.}

\item{local_test}{Logical. If \code{TRUE} and \code{global = FALSE}, local
permutation tests are performed for each raster cell. If \code{FALSE},
local SSIM layers are returned without p-values. Default is \code{FALSE}.}

\item{R}{Integer. Number of permutations used for statistical inference.
Default is \code{1000}.}

\item{fdr}{Logical. If \code{TRUE}, p-values are adjusted using the
Benjamini-Hochberg false discovery rate correction. Default is \code{TRUE}.}

\item{alpha}{Numeric. Significance level used to determine significant SSIM
values. Default is \code{0.05}.}

\item{seed}{Optional integer. Random seed used for permutation testing.
Default is \code{NULL}.}
}
\value{
If \code{global = TRUE} and \code{do_test = FALSE}, a list containing a summary
table of global SSIM, SIM, SIV, and SIP values is returned.

If \code{global = TRUE} and \code{do_test = TRUE}, a list containing the
summary table, permutation null distributions, p-values, and optionally
q-values is returned.

If \code{global = FALSE} and \code{do_test = FALSE}, a
\code{terra::SpatRaster} object with four layers is returned:
\code{SSIM}, \code{SIM}, \code{SIV}, and \code{SIP}.

If \code{global = FALSE}, \code{do_test = TRUE}, and
\code{local_test = TRUE}, a \code{terra::SpatRaster} object is returned with
local SSIM component layers, p-value layers, optionally q-value layers, and
significance layers.
}
\description{
This function calculates the Structural Similarity Index Measure (SSIM)
between two raster images. It can return either global SSIM summaries or
local SSIM raster layers. Optional permutation-based statistical inference
can also be performed for global or local SSIM values.
}
\details{
The SSIM index is calculated as the product of three components:
similarity in mean intensity, similarity in variance, and similarity in
spatial pattern. These components are returned as \code{SIM}, \code{SIV}, and
\code{SIP}, respectively.

The local SSIM calculation uses a square moving window with size
\code{(2 * w + 1) x (2 * w + 1)}.

When \code{do_test = TRUE}, permutation testing is performed by randomly
permuting the values of \code{map2} over the overlapping non-\code{NA} cells
and recalculating SSIM values.
}
\examples{
\dontrun{
library(terra)

# Global SSIM summary
g1 <- ssim_raster(
  map1,
  map2,
  global = TRUE,
  w = 1
)

# Local SSIM raster layers
l1 <- ssim_raster(
  map1,
  map2,
  global = FALSE,
  w = 1
)

# Local SSIM with permutation test
l2 <- ssim_raster(
  map1,
  map2,
  global = FALSE,
  w = 1,
  do_test = TRUE,
  local_test = TRUE,
  R = 999
)
}

}
